/*

Copyright ©1999 Mars Saxman
All Rights Reserved

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


Branchfractal Generator

Creates an iterative fern or tree like branching fractal.
Pixel values are calculated based on distance from the nearest
stem. This is one of the more complex algorithms. It would be nice
to make it even more complicated by adding a third dimension...
but that's off in the future somewhere.

*/

void* BranchfracInit(void);
void BranchfracExit(void* refcon);
float Branchfrac(float h, float v, void* refcon);
