/*

Copyright ©1999 Mars Saxman
All Rights Reserved

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


The rasterlib combines the pixmap, greymap, and transformation libraries.
It is just a convenient "one-stop-shopping" header.

*/

#ifndef __starfish_rasterlib__
#define __starfish_rasterlib__ 0

#include <stdlib.h>
#include "rasterliberrs.h"
#include "pixmap.h"
#include "greymap.h"
#include "bufferxform.h"

#endif //__starfish_rasterlib__
